# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/5/21 2:23 PM
# LAST MODIFIED ON:
# AIM:

import unittest
from sentence_spliter.logic_graph_en import long_cuter_en
from sentence_spliter.automata.sequence import EnSequence
from sentence_spliter.automata.state_machine import StateMachine


class test_spliter(unittest.TestCase):

    # @unittest.skip('pass')
    def test_bug_2021_5_21(self):
        paragraph = "\"There's -- got -- to -- be -- a -- shotcut,\" Ron pathed as they climbed their seventh long " \
                    "staircase and emerged on an unfamiliar landing, where there was nothing but a large painting of a bare" \
                    "stretch of grass hanging on the stone wall."
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        expect = ['"There\'s -- got -- to -- be -- a -- shotcut,"',
                  ' Ron pathed as they climbed their seventh long staircase and emerged on an '
                  'unfamiliar landing,',
                  ' where there was nothing but a large painting of a barestretch of grass '
                  'hanging on the stone wall.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_21_2(self):
        paragraph = "\"And Professor Lupin stepped over you, and waled toward the dementor, and pulled out his wand,\" " \
                    "said Hermione, \" and he said, 'None of us is hiding Sirius Black under our cloaks. Go.' But the " \
                    "dementor didn't move, so Lupin muttered something, and a silvery thing shot out of his wand at it, " \
                    "and itturned around and sort of glided away ....\""
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['"And Professor Lupin stepped over you, and waled toward the dementor, and '
                  'pulled out his wand," said Hermione,',
                  ' " and he said, \'None of us is hiding Sirius Black under our cloaks. Go.\' ',
                  "But the dementor didn't move, so Lupin muttered something, and a silvery "
                  'thing shot out of his wand at it, and itturned around and sort of glided '
                  'away ...."']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_21_3(self):
        paragraph = "noticed that they could slied through gaps that Uncle Vernon's new company car certainly " \
                    "couldn't have managed. They reached King's Cross with twenty minutes to spare; the Ministry" \
                    "drivers found them trolleys, unloaded their trunks, touched their hats in salute ti Mr. Weasley," \
                    "and drove away, somehow managing to jump to the head of an unmoving line at the traffic lights."

        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [
            "noticed that they could slied through gaps that Uncle Vernon's new company car certainly couldn't have managed.",
            " They reached King's Cross with twenty minutes to spare;",
            ' the Ministrydrivers found them trolleys, unloaded their trunks, touched their hats in salute ti Mr. Weasley,and drove away,',
            ' somehow managing to jump to the head of an unmoving line at the traffic lights.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_21_4(self):
        paragraph = "\"The sooner we get on the train, the better,\" he said. \"At least I can get away from" \
                    "Percy at Hogwarts. Now he's accusing me of dripping tea on his photo of Penelop Clearwarter." \
                    "You Know,\" Ron grimaced, \"his girlfriend. She's hidden her face under the frame because her nose " \
                    "gone all blotchy....\""
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['"The sooner we get on the train, the better," he said.',
                  ' "At least I can get away fromPercy at Hogwarts. Now he\'s accusing me of dripping tea on his photo of Penelop Clearwarter.You Know," Ron grimaced,',
                  ' "his girlfriend. She\'s hidden her face under the frame because her nose gone all blotchy...."']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_24_1(self):
        paragraph = "Harry had not chance to speak to Ron or Hermione in the chaos of leaving; they were too busey heaving all thier trunks down the Leaky Cauldron's " \
                    "narrow staircase and piling them up near the door, with Hedwig and Hermes, Percy's screech owl, perched on top in thier cages."
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['Harry had not chance to speak to Ron or Hermione in the chaos of leaving;',
                  " they were too busey heaving all thier trunks down the Leaky Cauldron's narrow staircase and piling them up near the door,",
                  " with Hedwig and Hermes, Percy's screech owl, perched on top in thier cages."]
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_24_2(self):
        paragraph = "I wept like a child. \"Dear mountains! my own beautiful lake! " \
                    "How do you welcome your wanderer? Your summits are clear; the sky and lake are blue and placid. Is this to prognosticate peace, or to mock at my unhappiness?\""
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['I wept like a child.',
                  ' "Dear mountains! my own beautiful lake! How do you welcome your wanderer?',
                  ' Your summits are clear; the sky and lake are blue and placid. Is this to '
                  'prognosticate peace, or to mock at my unhappiness?"']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_24_3(self):
        paragraph = "\"Not at all. The 'G' with the small 't' stands for 'Gesellschaft,' which is the German for 'Company" \
                    ".' It is a customary contraction like our 'Co.' 'P,' of course, stands for 'Papier.' Now for the " \
                    "'Eg.' Let us glance at our Continental Gazetteer.\" He took down a heavy brown volume from his " \
                    "shelves. \"Eglow, Eglonitz--here we are, Egria. It is in a German-speaking country--in Bohemia, " \
                    "not far from Carlsbad. 'Remarkable as being the scene of the death of Wallenstein, and for its " \
                    "numerous glass-factories and paper-mills.' Ha, ha, my boy, what do you make of that?\" His " \
                    "eyes sparkled, and he sent up a great blue triumphant cloud from his cigarette."

        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [
            '"Not at all. The \'G\' with the small \'t\' stands for \'Gesellschaft,\' which is the German for \'Company.\'',
            ' It is a customary contraction like our \'Co.\' \'P,\' of course, stands for \'Papier.\' Now for the \'Eg.\' Let us glance at our Continental Gazetteer."',
            ' He took down a heavy brown volume from his shelves.',
            ' "Eglow, Eglonitz--here we are, Egria. It is in a German-speaking country--in Bohemia, not far from Carlsbad.',
            " 'Remarkable as being the scene of the death of Wallenstein, and for its numerous glass-factories and paper-mills.' Ha, ha, my boy, what do you make of that?\"",
            ' His eyes sparkled, and he sent up a great blue triumphant cloud from his cigarette.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_24_4(self):
        paragraph = '''
I wept like a child. "Dear mountains! my own beautiful lake! how do you welcome your wanderer? Your summits are clear; the sky and lake are blue and placid. Is this to prognosticate peace, or to mock at my unhappiness?"
        '''

        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['\nI wept like a child.',
                  ' "Dear mountains! my own beautiful lake! how do you welcome your wanderer?',
                  ' Your summits are clear; the sky and lake are blue and placid. Is this to '
                  'prognosticate peace, or to mock at my unhappiness?"\n'
                  '        ']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_1(self):
        paragraph = '''"I was still balancing the matter in my mind when a hansom cab drove up to Briony Lodge, and a gentleman sprang out. He was a remarkably handsome man, dark, aquiline, and moustached, evidently the man of whom I had heard. He appeared to be in a great hurry, shouted to the cabman to wait, and brushed past the maid who opened the door with the air of a man who was thoroughly at home.'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [
            '"I was still balancing the matter in my mind when a hansom cab drove up to Briony Lodge, and a gentleman sprang out.',
            ' He was a remarkably handsome man, dark, aquiline, and moustached, evidently the man of whom I had heard.',
            ' He appeared to be in a great hurry, shouted to the cabman to wait,',
            ' and brushed past the maid who opened the door with the air of a man who was thoroughly at home.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_2(self):
        paragraph = '''"He was in the house about half an hour, and I could catch glimpses of him in the windows of the sitting-room, pacing up and down, talking excitedly, and waving his arms. Of her I could see nothing. Presently he emerged, looking even more flurried than before. As he stepped up to the cab, he pulled a gold watch from his pocket and looked at it earnestly, 'Drive like the devil,' he shouted, 'first to Gross & Hankey's in Regent Street, and then to the Church of St. Monica in the Edgeware Road. Half a guinea if you do it in twenty minutes!' '''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['"He was in the house about half an hour,',
                  ' and I could catch glimpses of him in the windows of the sitting-room, pacing up and down, talking excitedly, and waving his arms.',
                  ' Of her I could see nothing. Presently he emerged, looking even more flurried than before.',
                  ' As he stepped up to the cab,',
                  ' he pulled a gold watch from his pocket and looked at it earnestly,',
                  " 'Drive like the devil,' he shouted, 'first to Gross & Hankey's in Regent Street,",
                  " and then to the Church of St. Monica in the Edgeware Road. Half a guinea if you do it in twenty minutes!' "]
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_3(self):
        paragraph = ''' "It may be so, or it may not. Mr. Holmes," said he, "but if you are so very sharp you ought to be sharp enough to know that it is you who are breaking the law now, and not me. I have done nothing actionable from the first, but as long as you keep that door locked you lay yourself open to an action for assault and illegal constraint." '''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [' "It may be so, or it may not. Mr. Holmes," said he,',
                  ' "but if you are so very sharp you ought to be sharp enough to know that it is you who are breaking the law now, and not me.',
                  ' I have done nothing actionable from the first, but as long as you keep that door locked you lay yourself open to an action for assault and illegal constraint." ']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_4(self):
        paragraph = ''' Commenting on the stereotypes that Asian Americans face in the US, John C. Yang President and Executive Director of the Asian Americans Advancing Justice (AAJC) told CNN, “Unfortunately, Asian Americans and Pacific Islanders often are invisible to the public. Or, where we are visible, it falls into a couple of different stereotypes. One stereotype is the so called ‘model minority’ — the suggestion that there are no issues that really affect the Asian American community.” '''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [' Commenting on the stereotypes that Asian Americans face in the US, John C. '
                  'Yang President and Executive Director of the Asian Americans Advancing '
                  'Justice (AAJC) told CNN,',
                  ' “Unfortunately, Asian Americans and Pacific Islanders often are invisible '
                  'to the public.',
                  ' Or, where we are visible, it falls into a couple of different stereotypes. '
                  'One stereotype is the so called ‘model minority’',
                  ' — the suggestion that there are no issues that really affect the Asian '
                  'American community.” ']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_5(self):
        paragraph = ''' In a statement on March 19, US President Joe Biden urged Congress to “swiftly pass the COVID-19 Hate Crimes Act, which would expedite the federal government’s response to the rise of hate crimes exacerbated during the pandemic, support state and local governments to improve hate crimes reporting, and ensure that hate crimes information is more accessible to Asian American communities.”'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [' In a statement on March 19, US President Joe Biden urged Congress to '
                  '“swiftly pass the COVID-19 Hate Crimes Act,',
                  ' which would expedite the federal government’s response to the rise of hate '
                  'crimes exacerbated during the pandemic,',
                  ' support state and local governments to improve hate crimes reporting, and '
                  'ensure that hate crimes information is more accessible to Asian American '
                  'communities.”']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_6(self):
        paragraph = '''Now, the former child star is being hailed for her big screen directorial debut in "One Night in Miami," adapted from Kemp Powers' stage play about a meeting between Cassius Clay, Jim Brown, Sam Cooke and Malcolm X.'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [
            'Now, the former child star is being hailed for her big screen directorial debut in "One Night in Miami,"',
            " adapted from Kemp Powers' stage play about a meeting between Cassius Clay, Jim Brown, Sam Cooke and Malcolm X."]
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_7(self):
        paragraph = '''The numbers don't lie: For the second consecutive year, the percentages of women directing top-grossing films increased, reaching "recent historic highs," while the overall percentages of women working in key behind-the-scenes roles remained relatively stable, according to a study by San Diego State University released in January.'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ["The numbers don't lie: For the second consecutive year, the percentages of "
                  'women directing top-grossing films increased, reaching "recent historic '
                  'highs,"',
                  ' while the overall percentages of women working in key behind-the-scenes '
                  'roles remained relatively stable, according to a study by San Diego State '
                  'University released in January.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_8(self):
        paragraph = '''Lydia declared herself satisfied. “Oh! yes. It would be much better to wait till Jane was well, and by that time most likely Captain Carter would be at Meryton again. And when you have given your ball,” she added, “I shall insist on their giving one also. I shall tell Colonel Forster it will be quite a shame if he does not.”'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['Lydia declared herself satisfied.',
                  ' “Oh! yes. It would be much better to wait till Jane was well,',
                  ' and by that time most likely Captain Carter would be at Meryton again.',
                  ' And when you have given your ball,” she added,',
                  ' “I shall insist on their giving one also. I shall tell Colonel Forster it '
                  'will be quite a shame if he does not.”']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_9(self):
        paragraph = '''Two common classifications of chips are logic and memory. Logic chips perform operations; these include microprocessors – the brains your mobile phone, laptop, etc. Memory chips store data, either temporarily or in longer term storage (such as solid state drives).'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['Two common classifications of chips are logic and memory.',
                  ' Logic chips perform operations; these include microprocessors – the brains '
                  'your mobile phone, laptop, etc.',
                  ' Memory chips store data, either temporarily or in longer term storage (such '
                  'as solid state drives).']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_10(self):
        paragraph = '''You don't predict the future; you imagine the future. So as a science fiction writer whose stories often take place years or even centuries from now, I've found that people are really hungry for visions of the future that are both colorful and lived in, but I found that research on its own is not enough to get me there.'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ["You don't predict the future; you imagine the future.",
                  ' So as a science fiction writer whose stories often take place years or even '
                  'centuries from now,',
                  " I've found that people are really hungry for visions of the future that are "
                  'both colorful and lived in,',
                  ' but I found that research on its own is not enough to get me there.']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_11(self):
        paragraph = '''We also have a social and relational crisis; we're in the valley. We're fragmented from each other, we've got cascades of lies coming out of Washington...'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ["We also have a social and relational crisis; we're in the valley.",
                  " We're fragmented from each other, we've got cascades of lies coming out of "
                  'Washington...']
        self.assertEqual(expect, acutal)

    # @unittest.skip('pass')
    def test_bug_2021_5_25_12(self):
        paragraph = '''Quidditch team: three Chasers, whose job it was to score goals by putting the Quaffle(a red, soccer-sized ball) through one of the fifity-foot-high hoops at each end of the field;two Beaters, who were equipped with heavy bats to repel the Budgers(two heavy black balls that zoomed around trying to attack the players); a Keeper who defended the goalposts, and the Seeker, who had the'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = [
            'Quidditch team: three Chasers, whose job it was to score goals by putting the Quaffle(a red, soccer-sized ball)',
            ' through one of the fifity-foot-high hoops at each end of the field;',
            'two Beaters, who were equipped with heavy bats to repel the Budgers(two heavy black balls that zoomed around trying to attack the players)',
            '; a Keeper who defended the goalposts, and the Seeker, who had the']
        self.assertEqual(expect, acutal)

    def test_bug_2021_5_25_13(self):
        paragraph = '''the dog in the shadows of Magnolia Crescent. . . Lavernder Browb claooed her hands on'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=2, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['the dog in the shadows of Magnolia Crescent. . .',
                  ' Lavernder Browb claooed her hands on']
        self.assertEqual(expect, acutal)

    # 这个是一个分复杂的例子
    def test_bug_2021_5_25_14(self):
        paragraph = '''"Yeh always wait fer the hippogriff ter make the firs' move," Hagrid cointinued. "It's polite, see? Yeh walk toward him, and yeh bow, an' yeh wait. If he bows back, yeh're allowed ter toych him. If he desn' bow, then get away from him sharpish,' cause those talons hurt.'''
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)
        expect = ['"Yeh always wait fer the hippogriff ter make the firs\' move," Hagrid cointinued.',
                  ' "It\'s polite, see? Yeh walk toward him, and yeh bow, an\' yeh wait.',
                  " If he bows back, yeh're allowed ter toych him. If he desn' bow, then get away from him sharpish,' cause those talons hurt."]
        self.assertEqual(expect, acutal)

    def test_bug_2021_5_26(self):
        paragraph = "In the new economy, computer science isn't an optional skill – it's a basic skill, right along with the three \"Dr.\" Steven Nine out of ten parents want it taught at their children's schools"
        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input)
        acutal = m_input.sentence_list()
        print(acutal)



# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang
# CREATED ON: 2020/8/7 3:54 PM
# LAST MODIFIED ON:
# AIM:
from abc import ABC
import re
from typing import Tuple, List

from .abc import Operation
from .sequence import StrSequence, EnSequence
from . import condition
from .weights_list import weights as cut_weights
import copy


class Indolent(Operation):
    def __init__(self, symbols: dict, name: str = None):
        super(Indolent, self).__init__('Indolent' if not name else name, symbols)

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_candidate()
        return state


class Normal(Operation):
    def __init__(self, symbols: dict, name=None):
        if not name:
            name = 'Normal'
        super(Normal, self).__init__(name, symbols)

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_sentence_list()
        return state


class CutPreIdx(Operation):
    def __init__(self, symbols: dict, name=None):
        super(CutPreIdx, self).__init__('CutPreIdx' if not name else name, symbols)

    def operate(self, state: StrSequence) -> StrSequence:

        if self.symbols['quotation_left'].match(state.current_value):
            state.quota_left = state.quota_left - 1 if state.quota_left > 0 else 1
        if self.symbols['quotation_en'].match(state.current_value):
            state.quota_en = state.quota_en - 1 if state.quota_en > 0 else 1
        state.v_pointer -= 1
        state.add_to_sentence_list()
        return state


class LongHandler(Operation):
    def __init__(self, symbols: dict, hard_max: int = 300, name=None):
        super(LongHandler, self).__init__('LongHandler' if not name else name, symbols)
        self.is_end_symbol = condition.IsEndSymbolZH(symbols)
        self.is_comma = condition.IsComma(symbols)
        self.is_book_close = condition.IsBookClose(symbols)
        self.is_barcket_close = condition.IsBracketClose(symbols)
        self.is_right_quota = condition.IsRightQuotaZH()

        self.hard_max = hard_max

    def operate(self, state: StrSequence) -> StrSequence:
        # new_candidate = []
        # - step 1. end_symbol 过滤
        temps_state = copy.copy(state)
        for i in range(temps_state.sentence_start, temps_state.v_pointer - 1)[::-1]:
            temps_state.v_pointer = i
            if self.is_right_quota(temps_state) or (
                    self.is_end_symbol(temps_state) and self.is_book_close(temps_state)):
                cut_id = i
                break
        else:
            temps_state = copy.copy(state)
            # - step 2. comma 过滤
            for i in range(temps_state.sentence_start, temps_state.v_pointer - 1)[::-1]:  # enumerate(state.candidate):
                temps_state.v_pointer = i
                if self.is_comma(temps_state):
                    cut_id = i
                    break
            else:
                cut_id = state.sentence_start + self.hard_max - 1
        if cut_id > state.v_pointer:
            # -- step 3. run forward -- #
            while state.v_pointer < state.length and not self.is_end_symbol(state):
                state.v_pointer += 1
        else:
            state.v_pointer = cut_id

        return state


class ShortHandler(Operation):

    def __init__(self, symbols: dict, name=None):
        super(ShortHandler, self).__init__('ShortHandler' if not name else name, symbols)

    def operate(self, state: StrSequence) -> StrSequence:
        state.add_to_candidate()
        return state


class LongHandlerEN(Operation):
    def __init__(self, symbols: dict, white_list: str, min_sentence_length: int = 5, name=None):
        super(LongHandlerEN, self).__init__('LongHandlerEN' if not name else name, symbols)
        self.is_end_symbol = condition.IsEndSymbolEN(symbols)
        self.is_comma = condition.IsComma(symbols)
        self.is_book_close = condition.IsBookClose(symbols)
        self.is_bracket_close = condition.IsBracketClose(symbols)
        self.is_quota_close = condition.IsQuoteClose(symbols)
        self.not_in_white_list = condition.NotInWhitelistDotEn(white_list)
        self.is_next_with_space = condition.IsNextWithSpace(symbols)

        self.min_sentence_length = min_sentence_length
        self.is_dialogue_gt = condition.DialogueIsGreaterThanEN(min_sentence_length)
        self.is_bracket_gt = condition.BracketIsGreaterThan(min_sentence_length, symbols)
        self.space = self.symbols['all_symbols']
        self.blank = re.compile('^\s+$')
        self.is_right_quota = condition.IsRightQuotaEn()
        self.is_left_quota = condition.IsLeftQuotaEn()
        self.is_s_left = condition.IsSQuota('left')
        self.is_s_right = condition.IsSQuota('right')
        self.is_s_en = condition.IsSQuota('en')
        self.is_sentence_dash = condition.IsSenetnecDash()

        self.pre_index = 0
        self.pre_sentence_start = 0

        self.quota_left = 0
        self.quota_en = 0

        self.s_quota_left = 0
        self.s_quota_en = 0

        self.bracket_left = 0
        self.bracket_right = 0

        # --- build weigths dict -- #
        weights = cut_weights.splitlines()
        self.weights = dict()
        self.max_key_size = 1
        for v in weights:
            if not v:
                continue
            v = v.split()
            sym = v[0]
            word = ' '.join(v[1:-1])
            if (len(v) - 2) > self.max_key_size:
                self.max_key_size = len(v) - 2
            w = int(v[-1])
            w = int(w)
            self.weights[sym + ' ' + word] = w

        # self.weights = {k: i for k, i in sorted(self.weights.items(), key=lambda x: x[1])}

    def init_var(self):
        self.pre_index = 0
        self.pre_sentence_start = 0
        self.quota_left = 0
        self.quota_en = 0
        self.bracket_left = 0
        self.bracket_right = 0
        self.s_quota_left = 0
        self.s_quota_en = 0

    def update_state_info(self, state: EnSequence, temp_state: EnSequence) -> EnSequence:
        state.quota_en = temp_state.quota_en
        state.bracket_left = temp_state.bracket_left
        state.quota_left = temp_state.quota_left
        state.bracket_right = temp_state.bracket_right
        state.s_quota_en = temp_state.s_quota_en
        state.s_quota_left = temp_state.s_quota_left
        return state

    def operate(self, state: EnSequence) -> EnSequence:
        self.init_var()
        temps_state = copy.copy(state)
        self.get_init_quota_bracket(state)

        cut_id, success = self.outside_dialog(temps_state)
        if success:
            state.v_pointer = min(cut_id, state.length - 1)
            state.add_to_sentence_list()
            state = self.update_state_info(state, temps_state)
            return state
        else:
            temps_state = copy.copy(state)
            cut_id, success = self.inside_dialog(temps_state)
            if success:
                state.v_pointer = min(cut_id, state.length - 1)
                state.add_to_sentence_list()
                state = self.update_state_info(state, temps_state)
                return state
        return state

    def inside_dialog(self, state: EnSequence) -> Tuple[int, bool]:
        # -- traverse all elements to find all comma -- #
        max_score = -1
        best_i = -1
        length = state.v_pointer - state.sentence_start
        half_len = length // 2
        sub_len = 0
        state.quota_en = abs(state.quota_en - self.quota_en)
        state.quota_left = abs(state.quota_left - self.quota_left)
        state.s_quota_en = abs(state.s_quota_en - self.s_quota_en)
        state.s_quota_left = abs(state.s_quota_left - self.s_quota_left)
        state.bracket_right = abs(state.bracket_right - self.bracket_right)
        state.bracket_left = abs(state.bracket_left - self.bracket_left)
        best_sub_len = 0
        for idx, i in enumerate(range(state.sentence_start, state.v_pointer - 1)):
            state.v_pointer = i
            state.update_quota()
            state.update_barcket()
            if not self.space.match(state.current_value):
                sub_len += 1
            if self.blank.match(state.current_value):
                continue

            if state.current_value in [':', '：'] and sub_len > self.min_sentence_length and self.blank.match(
                    state[i + 1][0]):
                return i, True

            if (self.is_right_quota(state) or self.is_s_right(
                    state) or (state.current_value in '）)' and self.is_next_with_space(state))) and sub_len > self.min_sentence_length:
                return i, True

            # if self.is_next_with_space(state) and self.is_end_symbol(state) and self.not_in_white_list(
            #         state) and sub_len > self.min_sentence_length:
            #     return i, True
            if state.current_value == ',' and self.blank.match(state.next_value) or \
                    (self.is_next_with_space(state) and (self.is_end_symbol(state) or self.symbols['semicolon'].match(
                        state.current_value)) and self.not_in_white_list(state) and sub_len > self.min_sentence_length):
                # -- 首字母大写优先切分 -- #
                # if state[min(i + 2, length)][0].isupper():
                #     best_i = i
                #     max_score = 10
                #     best_sub_len = sub_len
                if self.symbols['all_quota'].match(state[min(i + 1, state.length - 1)]):
                    weight = 0.5 + 1 - abs(idx - half_len) / half_len
                    if weight > max_score:
                        best_i = min(i + 1, state.length - 1)
                        max_score = weight
                        best_sub_len = sub_len
                else:
                    # -- 在权重层里寻找合理的切句位置 -- #
                    for n_words in range(self.max_key_size):
                        key = ' '.join([state[min(i + ii * 2, state.length - 1)].lower() for ii in range(n_words + 2)])
                        postion_penalty = 1 - abs(idx - half_len) / half_len
                        weight = self.weights.get(key, 5) / 10 + postion_penalty
                        if self.is_end_symbol(state):
                            weight += 1
                        # weight = postion_penalty
                        if weight > max_score:
                            best_i = i
                            max_score = weight
                            best_sub_len = sub_len
        if max_score > 0 and best_sub_len > self.min_sentence_length:
            return best_i, True
        else:
            return best_i, False

    def outside_dialog(self, state: EnSequence) -> Tuple[int, bool]:
        org_pointer = state.v_pointer
        sub_len = 0
        state.quota_en = abs(state.quota_en - self.quota_en)
        state.quota_left = abs(state.quota_left - self.quota_left)
        state.s_quota_en = abs(state.s_quota_en - self.s_quota_en)
        state.s_quota_left = abs(state.s_quota_left - self.s_quota_left)
        state.bracket_right = abs(state.bracket_right - self.bracket_right)
        state.bracket_left = abs(state.bracket_left - self.bracket_left)
        for i in range(state.sentence_start, org_pointer):
            state.v_pointer = i
            state.update_quota()
            state.update_barcket()
            next_state = self.look_forward_state(state)
            if self.blank.match(state.current_value):
                continue
            if not self.space.match(state.current_value):
                sub_len += 1

            if self.is_next_with_space(state) and self.is_end_symbol(state) and self.not_in_white_list(
                    state) and self.is_quota_close(
                state) and self.is_bracket_close(state) and (
                    sub_len > self.min_sentence_length or self.is_left_quota(next_state) or self.is_s_quota_left(
                next_state)):
                return i, True

            if (self.is_sentence_dash(state) or self.symbols['semicolon'].match(
                    state.current_value)) and self.is_quota_close(state) and self.is_bracket_close(
                state) and sub_len > self.min_sentence_length:  # dash 在对话里面可能会有 长度的意思
                return i, True

            elif self.is_right_quota(state) and sub_len > self.min_sentence_length and self.is_dialogue_gt(state) and \
                    (self.is_next_with_space(state) or state.v_pointer == state.length - 1) and not self.symbols[
                'comma'].match(self.look_backward_value(state)):
                return min(i + 1, state.length - 1), True

            elif (self.is_left_quota(next_state) or self.is_s_left(next_state)) and sub_len > self.min_sentence_length \
                    and self.symbols['comma'].match(self.look_backward_value(next_state)):
                return i, True

            elif self.symbols['bracket_right'].match(
                    state.current_value) and sub_len > self.min_sentence_length and self.is_bracket_gt(state):
                return min(i, state.length - 1), True

            elif self.is_s_quota_right(state) and state.pre_value[
                -1] not in ['s', 'n'] and sub_len > self.min_sentence_length and self.is_dialogue_gt(state) and \
                    (self.is_next_with_space(state) or state.v_pointer == state.length - 1) and not self.symbols[
                'comma'].match(state.pre_value):
                return min(i + 1, state.length - 1), True
            else:
                continue
        else:
            state.v_pointer = org_pointer
            return -1, False

    def look_backward_value(self, state: EnSequence):
        for i in range(state.v_pointer)[::-1]:
            if not self.blank.match(state[i]):
                return state[i]
        else:
            return state.current_value

    def look_forward_state(self, state: EnSequence) -> EnSequence:
        out_state = copy.copy(state)
        for i in range(state.v_pointer + 1, state.length):
            out_state.v_pointer = i
            out_state.update_quota()
            out_state.update_barcket()
            out_state.update_bookmark()
            if not self.blank.match(state[i]):
                return out_state
        else:
            return out_state

    def get_init_quota_bracket(self, state: EnSequence):
        org_pointer = state.v_pointer
        start = state.sentence_start if self.pre_sentence_start != state.sentence_start else self.pre_index
        end_point = min(org_pointer + 1, state.length - 1)
        for i in range(start, end_point):
            state.v_pointer = i
            # double quota
            if self.symbols['quotation_left'].match(state[i]):
                self.quota_left += 1
            if self.symbols['quotation_right'].match(state[i]):
                self.quota_left = 0
                self.quota_en = 0
            if self.symbols['quotation_en'].match(state[i]):
                self.quota_en += 1
            # single quota
            if self.is_s_en(state):
                self.s_quota_en += 1
            if self.is_s_left(state):
                self.s_quota_left += 1
            if self.is_s_right(state):
                self.s_quota_left = 0
                self.s_quota_en = 0
            # bracket
            if self.symbols['bracket_left'].match(state[i]):
                self.bracket_left += 1
            if self.symbols['bracket_right'].match(state[i]):
                self.bracket_right += 1
        # self.pre_index = org_pointer
        self.pre_sentence_start = state.sentence_start

    def is_s_quota_right(self, state):
        if self.is_s_right(state):
            return True
        if self.is_s_en(state) and state.s_quota_en % 2 == 0:
            return True
        return False

    def is_s_quota_left(self, state):
        if self.is_s_left(state):
            return True
        if self.is_s_en(state) and state.s_quota_en % 2 != 0:
            return True
        return False


class EndState(Operation):
    def __init__(self, symbols: dict, name=None):
        super(EndState, self).__init__('EndState' if not name else name, symbols)

    def operate(self, state: StrSequence) -> StrSequence:
        if state.sentence_list and state.sentence_start >= state.length or state.length == 0:
            return state
        else:
            state.add_to_sentence_list()
            return state


# =========================== #
#      Special Condition      #
# =========================== #

class SpecialOperation(Operation, ABC):
    pass


class CleanNode(SpecialOperation):
    def __init__(self, node_list: List[Operation], name=None):
        super(CleanNode, self).__init__('EndState' if not name else name, None)
        self.node_list = node_list

    def operate(self, state: StrSequence) -> StrSequence:
        for node in self.node_list:
            node.init_var()

        # -- end state -- #
        if state.sentence_list and state.sentence_start >= state.length or state.length == 0:
            return state
        else:
            state.add_to_sentence_list()
            return state



# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang 
# CREATED ON: 2020/8/16 5:34 PM
# LAST MODIFIED ON: 2020/10/27 6:07 PM
# AIM:
import sys
from typing import Union, List
import re

from .symbols import SYMBOLS, SYMBOLS_EN


def print_percent(current: int, max: int, header: str = ''):
    percent = float(current) / max * 100
    sys.stdout.write("\r{0}{1:.3g}%".format(header, percent))


class StrSequence:
    def __init__(self, str_block: str, verbose=None, symbols=SYMBOLS):
        self.value = str_block
        # -- pointers -- #
        self.v_pointer = -1
        self.sentence_start = 0
        # -- results -- #
        self.sentence_list_idx = []
        self.length = len(self.value)
        self.verbose = verbose
        if verbose is None and len(self.value) > 50000:
            self.verbose = True

        # -- other pointers -- #
        self.bracket_left = 0
        self.bracket_right = 0
        self.quota_left = 0
        self.quota_en = 0
        self.book_left = 0
        self.book_right = 0
        self.symbols = symbols

    # -- magic method -- #
    def __str__(self):
        return self.value[self.sentence_start:self.v_pointer]

    def __getitem__(self, item: int):
        return self.value[item]

    @property
    def pre_value(self):
        return self[max(0, self.v_pointer - 1)]

    @property
    def next_value(self):
        return self[min(self.length - 1, self.v_pointer + 1)]

    @property
    def candidate_len(self):
        return self.v_pointer - self.sentence_start + 1

    @property
    def current_value(self):
        if self.value:
            return self.value[self.v_pointer]
        else:
            return ''

    def sentence_list(self):
        out = []
        for v in self.sentence_list_idx:
            if v[0] < v[1]:
                if isinstance(self.value, str):
                    out.append(self.value[v[0]:v[1]])
                else:
                    sentence = ''.join(self.value[v[0]:v[1]])
                    if sentence:
                        out.append(sentence)
        return out

    def at_end_pos(self):
        return self.v_pointer >= self.length

    def add_to_candidate(self):
        self.v_pointer = min(self.v_pointer + 1, self.length)
        # -- update pointer -- #
        self.update_barcket()
        self.update_quota()
        self.update_bookmark()
        if self.verbose:
            print_percent(self.v_pointer, self.length, 'process cutting ')

    def add_to_sentence_list(self):
        self.sentence_list_idx.append((self.sentence_start, min(self.length, self.v_pointer + 1)))
        self.sentence_start = self.v_pointer + 1

    def reach_right_end(self):
        if self.v_pointer >= self.length - 1:
            self.v_pointer = self.length - 1
            return True
        else:
            return False

    # --- update pointer --- #
    def update_barcket(self):
        key = self.current_value
        if self.symbols['bracket_left'].match(key):
            self.bracket_left += 1
        if self.symbols['bracket_right'].match(key):
            self.bracket_right += 1

    def reset_bracket(self):
        self.bracket_left = 0
        self.bracket_right = 0

    def update_quota(self):
        word = self.current_value
        if self.symbols['quotation_en'].match(word):
            self.quota_en += 1
        if self.symbols['quotation_left'].match(word):
            self.quota_left += 1
        if self.symbols['quotation_right'].match(word):
            self.quota_en = 0
            self.quota_left = 0

    def reset_quota(self):
        self.quota_en = 0
        self.quota_left = 0

    def update_bookmark(self):
        word = self.current_value
        if self.symbols['book_left'].match(word):
            self.book_left += 1
        if self.symbols['book_right'].match(word):
            self.book_right += 1

    def reset_bookmark(self):
        self.book_left = 0
        self.book_right = 0


class EnSequence(StrSequence):
    def __init__(self, str_block: str, verbose=None, symbols=SYMBOLS_EN):
        super().__init__(str_block, verbose, symbols)
        self.value = self.tokenize(str_block)
        self.length = len(self.value)
        self.__candidate_len = 0
        self.s_quota_en = 0
        self.s_quota_left = 0
        self.is_blank = re.compile('^\s+')

    def tokenize(self, str_block: str):
        pattern: re.Pattern = self.symbols['all_symbols']
        return [v for v in pattern.split(str_block) if v]

    def __is_squota(self, re):
        if re.match(self.current_value):
            if self.v_pointer == 0:
                return True
            elif self.v_pointer == self.length - 1:
                return True
            elif self.is_blank.match(self.pre_value) or self.is_blank.match(self.next_value):
                if self.pre_value not in ['s','n']:
                    return True
            else:
                return False
        return False

    def update_quota(self):
        word = self.current_value
        if self.symbols['quotation_en'].match(word):
            self.quota_en += 1
        if self.symbols['quotation_left'].match(word):
            self.quota_left += 1
        if self.symbols['quotation_right'].match(word):
            self.quota_en = 0
            self.quota_left = 0
        # -- single quota -- #
        if self.__is_squota(self.symbols['s_quota_left']):
            self.s_quota_left += 1
        if self.__is_squota(self.symbols['s_quota_en']):
            self.s_quota_en += 1
        if self.__is_squota(self.symbols['quotation_right']):
            self.s_quota_left = 0
            self.s_quota_en = 0

    def reset_s_quota(self):
        self.s_quota_en = 0
        self.s_quota_left = 0

    @property
    def candidate_len(self):
        return self.__candidate_len

    def add_to_candidate(self):
        self.v_pointer = min(self.v_pointer + 1, self.length - 1)
        if not self.symbols['all_symbols'].match(self.current_value):
            self.__candidate_len += 1

        # -- update pointer -- #
        self.update_barcket()
        self.update_quota()
        self.update_bookmark()
        if self.verbose:
            print_percent(self.v_pointer, self.length, 'process cutting')

    def add_to_sentence_list(self):
        self.sentence_list_idx.append((self.sentence_start, min(self.length, self.v_pointer + 1)))
        self.sentence_start = self.v_pointer + 1
        self.__candidate_len = 0



# -*- coding:utf-8 -*-
# CREATED BY: bohuai jiang 
# CREATED ON: 2020/8/12 4:42 PM
# LAST MODIFIED ON:
# AIM:
import re

SYMBOLS = {
    'quotation_left': re.compile('([“])'),
    'quotation_en': re.compile('(["])'),
    'quotation_right': re.compile('([”])'),
    'bracket_left': re.compile('([<{\[\(（【「])'),
    'bracket_right': re.compile('([\)\]}」】）>])'),
    'book_left': re.compile('([《])'),
    'book_right': re.compile('([》])'),
    'comma': re.compile('([,，;])'),
    'end_symbols': re.compile('([?\!…;？！。\.])'),
    'en_dot': re.compile('\.'),
    'all_symbols': re.compile('([\]\(【\?】。,！[”，\!\.「<>？"（“…《）}》」;:：\){\)\s])')
}


SYMBOLS_EN = {
    'quotation_left': re.compile('([“])'),
    's_quota_left': re.compile('([‘])'),
    'quotation_en': re.compile('(["])'),
    's_quota_en':  re.compile("(['])"),
    'quotation_right': re.compile('([”])'),
    's_quota_right': re.compile('([’])'),
    'all_s_quota': re.compile('([‘’\'])'),
    'all_quota': re.compile('([“"”])'),
    'bracket_left': re.compile('([<{\[\(（【「])'),
    'bracket_right': re.compile('([\)\]}」】）>])'),
    'book_left': re.compile('([《])'),
    'book_right': re.compile('([》])'),
    'comma': re.compile('([,，])'),
    'end_symbols': re.compile('([?\!…？！。\.])'),
    'en_dot': re.compile('\.'),
    'dash': re.compile('—'),
    'short_dash': re.compile('-'),
    'semicolon': re.compile('([;；])'),
    'all_symbols': re.compile('([\]\(【\?】。,！[”，\!\.「<>？"（“…《）}》」;；:：\'‘’\-\—\){\)\s])')
}
